<!-- -*- mode: markdown; mode: flyspell -*- -->

# ONMULTI

Simple tool to run a command on one or more remote hosts. Uses SSH to
connect to hosts and at the remote side optionally sudo to select the
executing user.

## Usage

```
onmulti: usage: onmulti [OPTIONS] <range> <remote-command> [..]
  where OPTIONS are:
    -c <config-file>        .. default is /etc/physical-config/bash_ranges.sh
    -h                      .. show this help
    -q                      .. quiet output
    -r <range>              .. additional range to use
    -s <sudo-user>          .. escalade privileges via sudo on remote host
    -S                      .. shortcut for -s root
    -u <ssh-users>          .. username to use logging into remote host
    -U                      .. shortcut for -u root
    -v                      .. increase verbose level
  Note: SSH and SUDO must work non interactive or it will fail.
```

### About Ranges

Ranges have names and one the following contents:

1. A macro: Any string, which is the name name of any range. Its value
   will be expanded in recursive fashion.

2. A range of numbers `<number>-<number>` will be expanded to a series
   of consecutive numbers. Prefix and suffix (see Configuration
   section) will be added to each number, if defined.

3. A number: Will expand to itself, with prefix and suffix being
   added, if defined.

4. Any other string will be used unmodified.

## Configuration

The configuration file must currently be a bash style file containing
the variable definitions described below. If no configuration file is
specified and the default configuration file is not found, onmulti
will still work but only accept hostnames as ranges.

### Range Macro Definitions

A bash associative array variable definition can be specified,
containing all ranges as values and names as keys.

### Prefix and Suffixes

A prefix for all hosts can be specified with
`BRE_HOST_PREFIX="my-prefix"` and a suffix for all hosts can be
specified with `BRE_HOST_SUFFIX="my-suffix"`.

### Example

The following example configuration shows a scenario where there are
three groups of workstation (red, green and blue) that have numbered
names (workstation1 .. workstation30) and a group called white
containing servers. The macros workstations and servers can be used to
address all workstations or servers. The macro all can be used to
refer to all hosts.

```
BRE_HOST_PREFIX=workstation
BRE_RANGE_MACROS=(
	[red]="1-10"
	[green]="11-20"
	[blue]="21-30"
	[workstations]="red green blue"
	[servers]="server1.localdomain server2.localdomain"
	[all]="workstations servers"
)
```

To run `date` on all green workstations do:
```
onmulti green date
```

To run `uname -a` on all workstations and servers do:
```
onmulti all uname -a
```

## Logo

The logo source is
[Wikipedia](https://en.wikipedia.org/wiki/File:StarNetwork.svg). The
original version was released into the public domain.

<!-- LocalWords: onmulti hostnames sudo -->
